%Alistair's code here
%24 Jan 05. Modified to use a value for lambda_i as well as lambda_Ach.

function [Qe_root, Qi_root, Ve_root, Vi_root] = steady_states_delV_lambda(...
			del_Vrest, lambda, lambda_i,graphFlag, Nsearch)
% Find steady states for Hasselmo model for sleep cycling.  Model ACh as
% increasing resting voltage.  Allow EPSP strength to vary (increases during
% slow-wave-sleep bursting; decreases during REM).

global H

H = init_hasselmo_globs;

% Step 1: Compute error residuals in Qe via Eqs 33.3 / 33.4  --------------
%         Qe_1 --> Qi_2 --> Qe_3 

offset = 0;
Qe_1 = linspace(0+offset, H.Qe_max-offset, Nsearch)';
err_Qe = Qe_resid(Qe_1, del_Vrest, lambda, lambda_i);

Qe_3 = Qe_1 + err_Qe;
Ve_1 = invQsige(Qe_1);
Ve_3 = invQsige(Qe_3);
err_Ve = Ve_3 - Ve_1;  	% voltage residual

% Step 2: Compute error residuals in Qi via Eqs 31.4 / 31.3  -------------- 
%         Qi_4 --> Qe_5 --> Qi_6 
Qi_4 = linspace(0+offset, H.Qi_max-offset, Nsearch)';
err_Qi = Qi_resid(Qi_4, del_Vrest, lambda, lambda_i);

Qi_6 = Qi_4 + err_Qi;
Vi_4 = invQsigi(Qi_4);
Vi_6 = invQsigi(Qi_6);
err_Vi = Vi_6 - Vi_4;  

% Step 3: Count the no. of [Qe, Qi] roots (look for sign-change in residuals)
chs_Qe = err_Qe(1:end-1) .* err_Qe(2:end);
chs_Qe_index = find(chs_Qe < 0);

chs_Qi = err_Qi(1:end-1) .* err_Qi(2:end);
chs_Qi_index = find(chs_Qi < 0);

[n_roots  n_roots_Qi] = deal(length(chs_Qe_index), length(chs_Qi_index));

% sanity check: Check that number of roots for Qe and Qi are equal!
if (n_roots ~= n_roots_Qi)
	[n_roots  n_roots_Qi]
	chs_Qe_index, chs_Qi_index
	warning('Unbalanced root count-- skipping this point!');
	n_roots = 0;
end

% Step 4: Form a bracketting interval for each root
if n_roots >= 1
	brack_Qe = [Qe_1(chs_Qe_index)  Qe_1(chs_Qe_index + 1)];
	brack_Qi = [Qi_4(chs_Qi_index)  Qi_4(chs_Qi_index + 1)];
	
	% Check for NaN at either end of bracket
	if any(isnan(brack_Qe(:))) | any(isnan(brack_Qi(:)))
		error('Detected NaN in a bracket!');
	end
end

% Step 5: Polish each root
Qe_root = NaN*ones(n_roots, 1);
Qi_root = NaN*ones(n_roots, 1);
for i = 1: n_roots
	Qe_root(i) = fzero('Qe_resid', brack_Qe(i,:), [], [], ...
        del_Vrest, lambda, lambda_i);
	Qi_root(i) = fzero('Qi_resid', brack_Qi(i,:), [], [], ...
        del_Vrest, lambda, lambda_i);
end

% Qe_root
% Qi_root
Ve_root = invQsige(Qe_root);
Vi_root = invQsigi(Qi_root);

%--------------------
if ~graphFlag
	return
end

% plot graphs of steady-state error residuals
figure(1); 
subplot(121);
	plot(Qe_1, err_Qe, 'r-', 'linewidth', 1); zoom on; grid on; hold on;
	plot(Qi_4, -err_Qi, 'b-', 'linewidth', 1); zoom on; grid on; 
	plot(Qe_root(:,1), 0*Qe_root(:,1), 'ko', 'linewidth', 1); zoom on; grid on;
	plot(Qi_root(:,1), 0*Qi_root(:,1), 'kx', 'linewidth', 1); zoom on; grid on; hold off;
axis([0 60 -10 10]);
	title('{\Delta}Q residuals:  red = {\Delta}Qe; blue = {\Delta}Qi', 'fontsize', 18);
	ylabel('{\Delta}Qe,i (s^{-1})', 'fontsize', 16);
	xlabel('Qe,i (s^{-1})', 'fontsize', 16);
subplot(122);
	plot(Ve_1, err_Ve, 'r-', 'linewidth', 1); zoom on; grid on; hold on;
	plot(Vi_4, -err_Vi, 'b-', 'linewidth', 1); zoom on; grid on;
	plot(Ve_root(:,1), 0*Ve_root(:,1), 'ko', 'linewidth', 1); zoom on; grid on;
	plot(Vi_root(:,1), 0*Vi_root(:,1), 'kx', 'linewidth', 1); zoom on; grid on; hold off;
	title('{\Delta}V residuals:  red = {\Delta}Ve; blue = {\Delta}Vi', 'fontsize', 18);
	ylabel('{\Delta}Ve,i  (mV)', 'fontsize', 16);
	xlabel('Ve,i  (s^{-1})', 'fontsize', 16);
axis([-70 -50 -5 5]);

if graphFlag, drawnow, end

return

%------------------------------------------------------------------------
function err_Qe = Qe_resid(Qe_1, del_Vrest, lambda, lambda_i)
% Combine calls to Eq333() and Eq334() into a single function to return
% error residual in Qe

Ve_1 = invQsige(Qe_1);
Qi_2 = Eq333(Qe_1, Ve_1, del_Vrest, lambda, lambda_i);

Vi_2 = invQsigi(Qi_2);
Qe_3 = Eq334(Qi_2, Vi_2, del_Vrest, lambda, lambda_i);

err_Qe = Qe_3 - Qe_1;

return

%------------------------------------------------------------------------
function err_Qi = Qi_resid(Qi_4, del_Vrest, lambda, lambda_i)
% Combine calls to Eq334() and Eq333() into a single function to return
% error residual in Qi

Vi_4 = invQsigi(Qi_4);
Qe_5 = Eq334(Qi_4, Vi_4, del_Vrest, lambda, lambda_i);

Ve_5 = invQsige(Qe_5);
Qi_6 = Eq333(Qe_5, Ve_5, del_Vrest, lambda, lambda_i);

err_Qi = Qi_6 - Qi_4;

return

%------------------------------------------------------------------------
function Qi_2 = Eq333(Qe_1, Ve_1, del_Vrest, lambda, lambda_i)
% Given Qe_1 and Ve_1, use Eq.33.3 to compute Qi_2.
% del_Vrest adjusts resting voltage; lambda scales EPSP

global H

psi_ee = Psi_ee(Ve_1);
psi_ie = Psi_ie(Ve_1);

numer = Ve_1 - H.Ve_rest - del_Vrest - ...
	 lambda * H.ge * psi_ee .* (H.Nee_ab * Qe_1 + H.phi_ee_sc);
denom = H.gi * lambda_i * psi_ie;
Qi_2 = (numer ./ denom - H.phi_ie_sc) / H.Nie_b;
Qi_2(Qi_2 < 0 | Qi_2 > H.Qi_max) = NaN;

return

%------------------------------------------------------------------------
function Qe_3 = Eq334(Qi_2, Vi_2, del_Vrest, lambda, lambda_i)
% Given Qi_2 and Vi_2, use Eq.33.4 to compute Qe_3.
% del_Vrest adjusts resting voltage (not used here); lambda scales EPSP

global H

psi_ei = Psi_ei(Vi_2);
psi_ii = Psi_ii(Vi_2);

numer = Vi_2 - H.Vi_rest - H.gi * lambda_i* psi_ii .* (H.Nii_b * Qi_2 + H.phi_ii_sc);
denom = lambda * H.ge * psi_ei;
Qe_3 = (numer ./ denom - H.phi_ei_sc) / (H.Nei_ab);
Qe_3(Qe_3 < 0 | Qe_3 > H.Qe_max) = NaN;

return

%------------------------------------------------------------------------
function [x_clean, y_clean] = filterNaN(x, y)
% Given vectors x and y, return the subset vectors than contain no NaN
% elements.  We treat the (x,y) as ordered pairs, and require that neither
% element of a pair contain an NaN.

valids = isfinite(x .* y);
[x_clean  y_clean] = deal(x(valids), y(valids));

return

%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------
function  invsig = invQsige(val)
% Inverse of excitatory sigmoid function; output in millivolts

global H
gain = pi/(sqrt(3) * H.sigma_e);
invsig = NaN*ones(size(val));
ok = find(val > 0 & val < H.Qe_max);

invsig(ok) = H.theta_e - (1/gain)*log(H.Qe_max./val(ok) - 1);
return

%------------------------------------------------------------------------
function  invsig = invQsigi(val)
% Inverse of inhibitory sigmoid function

global H
gain = pi/(sqrt(3) * H.sigma_i);
invsig = NaN*ones(size(val));
ok = find(val > 0 & val < H.Qi_max);

invsig(ok) = H.theta_i - (1/gain)*log(H.Qi_max./val(ok) - 1);
return

%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------


%------------------------------------------------------------------------
function QC_plot_sigmoids
% Quality control check: Plot sigmoids

global H
H = init_hasselmo_globs;

num_pts = 1e3;

Ve = linspace(invQsige(1e-10), invQsige(H.Qe_max - 1e-10), num_pts)';
Qe = Qsige(Ve);

Vi = linspace(invQsigi(1e-10), invQsigi(H.Qi_max - 1e-10), num_pts)';
Qi = Qsigi(Vi);

figure(1); 
subplot(111);
	plot(Ve, Qe, '-r', Vi, Qi, '-b'); zoom on; grid on;
	title('Sigmoids: Firing rate vs Soma voltage (mV)', 'fontsize', 16); 
	legend('Qe', 'Qi');
return

%------------------------------------------------------------------------
function QC_deriv_sigmoids
% Quality control check for d_Qsige() and d_Qsigi()

global H

H = init_hasselmo_globs;
num_pts = 1e3;

Ve = linspace(invQsige(1e-10), invQsige(H.Qe_max - 1e-10), num_pts)';
Qe = Qsige(Ve);
dQe   = d_Qsige(Ve);				% analytic derivative
dQe_2 = cdiff(Qe)/(Ve(2) - Ve(1));	% numerical derivative

Vi = linspace(invQsigi(1e-10), invQsigi(H.Qi_max - 1e-10), num_pts)';
Qi = Qsigi(Vi);
dQi   = d_Qsigi(Vi);				% analytic derivative
dQi_2 = cdiff(Qi)/(Vi(2) - Vi(1));	% numerical derivative

figure(1); 
subplot(221);
	plot(Ve, dQe, '-r', Ve, dQe_2, '-b'); zoom on; grid on;
	title('Q_e deriv: analytic (red) vs numerical (blue)', 'fontsize', 16); 
subplot(222);
	plot(Ve, dQe - dQe_2, '-r'); zoom on; grid on;
	title('Q_e deriv: Error residual', 'fontsize', 16); 
subplot(223);
	plot(Vi, dQi, '-r', Vi, dQi_2, '-b'); zoom on; grid on;
	title('Q_i deriv: analytic (red) vs numerical (blue)', 'fontsize', 16); 
subplot(224);
	plot(Vi, dQi - dQi_2, '-b'); zoom on; grid on;
	title('Q_i deriv: Error residual', 'fontsize', 16); 

return

%------------------------------------------------------------------------
function b = fzero(FunFcn,x,tol,trace,varargin)
%FZERO  Find zero of function of one variable. 
%   FZERO(F,X) tries to find a zero of F.  F is a string containing 
%   the name of a real-valued function of a single real variable.   
%   The value returned is near a point where F changes sign, or NaN 
%   if the search fails.  
%
%   FZERO(F,X), where X is a vector of length 2, assumes X is an 
%   interval where the sign of F(X(1)) differs from the sign of F(X(2)).
%   An error occurs if this is not true.  Calling FZERO with an interval  
%   guarantees FZERO will return a value near a point where F changes 
%   sign.
%
%   FZERO(F,X), where X is a scalar value, uses X as a starting guess. 
%   FZERO looks for an interval containing a sign change for F and 
%   containing X.  If no such interval is found, NaN is returned.  
%   In this case, the search terminates when the search interval 
%   is expanded until an Inf, NaN, or complex value is found. 
%
%   FZERO(F,X,TOL) sets the relative tolerance for the convergence test.  
%   FZERO(F,X,TOL,TRACE) displays information at each iteration when
%   TRACE is nonzero.
%   FZERO(F,X,TOL,TRACE,P1,P2,...) allows for additional arguments
%   which are passed to the function, F(X,P1,P2,...).  Pass an empty
%   matrix for TOL or TRACE to use the default value.
%
%   Examples
%       fzero('sin', 3) returns pi. Note the quotes around sin.  
%       Ordinarily, functions are defined in M-files.
%       fzero('abs(x)+1', 1) returns NaN since this function does
%       not change sign anywhere on the real axis (and does not have
%       a zero as well).
%       fzero('sin', 3, [], 1) returns pi, uses the default tolerance,
%       and displays iteration information.
%
%   See also ROOTS.

%   Copyright (c) 1984-96 by The MathWorks, Inc.
%   $Revision: 5.10 $  $Date: 1996/10/28 22:13:28 $

%  This algorithm was originated by T. Dekker.  An Algol 60 version,
%  with some improvements, is given by Richard Brent in "Algorithms for
%  Minimization Without Derivatives", Prentice-Hall, 1973.  A Fortran
%  version is in Forsythe, Malcolm and Moler, "Computer Methods
%  for Mathematical Computations", Prentice-Hall, 1976.

% Initialization

if nargin < 3 | isempty(tol), tol = eps; end
if nargin < 4 | isempty(trace), trace = 0; end
if trace 
  header = ' Func evals      x            f(x)          Procedure';
  step=' ';
  count = 0;
end
if  (~isfinite(x))
   error('Second argument must be finite.')
end

% Convert to inline function as needed.
% 26 July 2004 mtw Comment out this next line to make it run in Matlab 7
%FunFcn = fcnchk(FunFcn,length(varargin));

% Interval input
if (length(x) == 2) 
  a = x(1); 
  b = x(2);
  fa = feval(FunFcn,a,varargin{:});
  fb = feval(FunFcn,b,varargin{:});
  if any(~isfinite([fa fb])) | any(~isreal([fa fb]))
     error('Function values at interval endpoints must be finite and real.')
  end
  if trace
    disp(header)
    data = [a fa]; step='       initial';
    count = count + 1;
    disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
    data = [b fb]; step = '       initial';
    count = count + 1;    
    disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
  end
  if (fa > 0) == (fb > 0)
    error('The function values at the interval endpoints must differ in sign.')
  end

% Starting guess scalar input
elseif (length(x) == 1)
  fx = feval(FunFcn,x,varargin{:});
  if fx == 0
    b = x;
    return
  elseif ~isfinite(fx) | ~isreal(fx)
    error('Function value at starting guess must be finite and real.');
  end
  if trace 
     disp(header)
     data = [x fx]; step='       initial';
     count = count + 1;     
     disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
  end
  if x ~= 0, 
    dx = x/50;
  else, 
    dx = 1/50;
  end

  % Find change of sign.
  twosqrt = sqrt(2); 
  a = x; fa = fx; b = x; fb = fx;
  
  while (fa > 0) == (fb > 0)
     dx = twosqrt*dx;
     a = x - dx;  fa = feval(FunFcn,a,varargin{:});
     if ~isfinite(fa) | ~isreal(fa)
       disperr(a,fa);
       b = NaN;
       return
     end
     if trace 
        data = [a fa];  step='       search';
        count = count + 1;
        disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
     end
     if (fa > 0) ~= (fb > 0)
        break
     end
     b = x + dx;  fb = feval(FunFcn,b,varargin{:});
     if ~isfinite(fb) | ~isreal(fb)
       disperr(b,fb);
       b = NaN;
       return
     end
     if trace 
        data = [b fb];  step='       search';
        count = count + 1;        
        disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
     end
   end % while
   if trace
     disp(' ')
     disp(['   Looking for a zero in the interval [', ...
                 num2str(a) , ', ', num2str(b), ']']);
     disp(' ')
   end
else
   error('Second argument must be of length 1 or 2.');
end % if (length(x) == 2

fc = fb;
% Main loop, exit from middle of the loop
while fb ~= 0
   % Insure that b is the best result so far, a is the previous
   % value of b, and c is on the opposite of the zero from b.
   if (fb > 0) == (fc > 0)
      c = a;  fc = fa;
      d = b - a;  e = d;
   end
   if abs(fc) < abs(fb)
      a = b;    b = c;    c = a;
      fa = fb;  fb = fc;  fc = fa;
   end

   % Convergence test and possible exit
   m = 0.5*(c - b);
   toler = 2.0*tol*max(abs(b),1.0);
   if (abs(m) <= toler) + (fb == 0.0), break, end

   % Choose bisection or interpolation
   if (abs(e) < toler) + (abs(fa) <= abs(fb))
   % Bisection
      d = m;  e = m;
      step='       bisection';
   else
   % Interpolation
      s = fb/fa;
      if (a == c)
      % Linear interpolation
         p = 2.0*m*s;
         q = 1.0 - s;
      else
      % Inverse quadratic interpolation
         q = fa/fc;
         r = fb/fc;
         p = s*(2.0*m*q*(q - r) - (b - a)*(r - 1.0));
         q = (q - 1.0)*(r - 1.0)*(s - 1.0);
      end;
      if p > 0, q = -q; else p = -p; end;
      % Is interpolated point acceptable
      if (2.0*p < 3.0*m*q - abs(toler*q)) * (p < abs(0.5*e*q))
         e = d;  d = p/q;
         step='       interpolation';
      else
         d = m;  e = m;
         step='       bisection';
      end;
   end % Interpolation

   % Next point
   a = b;
   fa = fb;
   if abs(d) > toler, b = b + d;
   else if b > c, b = b - toler;
        else b = b + toler;
        end
   end
   fb = feval(FunFcn,b,varargin{:});
   if trace 
      data = [b fb];  
      count = count + 1;      
      disp([sprintf('%5.0f   %13.6g %13.6g ',count, data), step])
   end
end % Main loop

%------------------------------------------------------------------
function disperr(y, fy)
%DISPERR Display an appropriate error message when FY is Inf, 
%   NaN, or complex.  Assumes Y is the value and FY is the function 
%   value at Y. If FY is neither Inf, NaN, or complex, it generates 
%   an error message.

if ~isfinite(fy)  % NaN or Inf detected
       disp('NaN or Inf function value encountered during ');
       disp('   search for an interval containing a sign change.');
       disp(['Function value at ', num2str(y),' is ',num2str(fy)]);
       disp('Aborting since no such interval was found.')
       disp('Check function or try again with a different starting value.')
elseif ~isreal(fy) % Complex value detected
       disp('Complex function value encountered during ');
       disp('   search for an interval containing a sign change.');
       disp(['Function value at ', num2str(y),' is ',num2str(fy)]);
       disp('Aborting since no such interval was found.')
       disp('Check function or try again with a different starting value.')
else
       error('Disperr called with invalid argument.')
end